import os
from werkzeug.utils import import_string
from logging.config import dictConfig
from flask import Flask
from celery import Celery


# Environment key to config flask instance path
ENV_INSTANCE_PATH = 'FLASK_INSTANCE_PATH'
# Environment key to config flask config module
ENV_CONFIG_MODULE = 'FLASK_CONFIG_MODULE'


def config_logger(app):
    if 'LOGGER' in app.config:
        dictConfig(app.config['LOGGER'])


def register_app(app):
    """
    Import blueprints module in REGISTERED_APP config.

    :param app: flask app
    """
    with app.app_context():
        for a in app.config['REGISTERED_APP']:
            bp = '{}.blueprints'.format(a)
            import_string(bp)


def create_app_by_config(conf=None, config_log=True, register=True):
    """
    Create flask app by config object.

    :param conf: config object
    :param config_log: whether to config logger
    :param register: whether to register apps
    :return: flask app
    """
    # check instance path
    instance_path = os.environ.get(ENV_INSTANCE_PATH) or None
    # create app
    app = Flask(__name__, instance_path=instance_path)
    # ensure the instance folder exists
    if app.instance_path:
        try:
            os.makedirs(app.instance_path, exist_ok=True)
        except OSError:
            pass
    # configure app
    if conf:
        app.config.from_object(conf)
    # config logger
    if config_log:
        config_logger(app)
    # register app
    if register:
        register_app(app)
    return app


def create_app(config_log=True, register=True):
    """
    Create flask app by config module.

    :param config_log: whether to config logger
    :param register: whether to register apps
    :return: flask app
    """
    config = os.environ.get(ENV_CONFIG_MODULE) or 'config'
    return create_app_by_config(conf=config, config_log=config_log, register=register)


def init_celery(app):
    cel = Celery(app.name)
    cel.conf.update(app.config['CELERY_CONFIG'])
    # register tasks
    with app.app_context():
        register_apps = app.config['REGISTERED_APP']
        for app in register_apps:
            import_string('{}.tasks'.format(app), silent=True)
    return cel



from platform import python_version
import click
from flask import __version__ as flask_version
from flask_board import __version__ as board_version
from flask_board.generator import guess_generator


def print_version(ctx, param, value):
    if not value or ctx.resilient_parsing:
        return
    click.echo('Python {}'.format(python_version()))
    click.echo('Flask {}'.format(flask_version))
    click.echo('Flask Board {}'.format(board_version))
    ctx.exit()


@click.command(short_help='Generate flask app by templates')
@click.argument('name')
@click.option('-d', '--directory', help='Project directory, default current directory')
@click.option('-t', '--template', help='Template name', default='default')
@click.option('-a', '--additional', help='Additional parameters send to generator', multiple=True)
@click.option('-s', '--skip', help='Skip prompts and use default', is_flag=True)
@click.option('--excludes', help='Exclude file patterns in template directory, comma separated')
@click.option('--excludes-dir', help='Exclude directory patterns in template directory, comma separated')
@click.option('--version', help='Show version', is_flag=True, callback=print_version, expose_value=False, is_eager=True)
def cli(name, directory, template, **kwargs):
    """
    Create Flask project by template.

    name: project name
    """
    generator = guess_generator(name=name, directory=directory, template=template)
    try:
        generator.run(**kwargs)
    except (FileNotFoundError, FileExistsError) as e:
        click.echo(e)



import os
from dotenv import load_dotenv


# Load env file
# The file path is set by environment FLASK_ENV_FILE
env_file_key = 'FLASK_ENV_FILE'
env_file = os.getenv(env_file_key) or '.env'
if os.path.exists(env_file):
    load_dotenv(env_file)


# All uppercase attributes will be load to flask config

SECRET_KEY = os.getenv('SECRET_KEY') or '{{ secret }}'
DEBUG = os.getenv('DEBUG') == 'on'
TESTING = os.getenv('TESTING') == 'on'

# Registered app packages
REGISTERED_APP = [
    'app'
]

# Logger configuration
log_level = os.getenv('LOG_LEVEL') or 'INFO'
LOGGER = {
    'version': 1,
    'formatters': {
        'default': {
            'format': '[%(asctime)s] %(levelname)s in %(module)s: %(message)s'
        },
        'detail': {
            'format': '[%(asctime)s] %(levelname)s in %(module)s: %(message)s'
        }
    },
    'handlers': {
        'console': {
            'class': 'logging.StreamHandler',
            'formatter': 'default',
            'level': log_level,
            'stream': 'ext://flask.logging.wsgi_errors_stream'
        },
        'file': {
            'class': 'logging.handlers.RotatingFileHandler',
            'formatter': 'detail',
            'filename': os.getenv('LOG_FILE') or 'app.log',
            'level': log_level,
            'maxBytes': 5000000,
            'backupCount': 5,
            'delay': True
        }
    },
    'root': {
        'level': log_level,
        'handlers': (os.getenv('LOG_HANDLERS') or 'console,file').split(',')
    }
}



import os
import shutil
import random
import fnmatch
import string
import click


def generate_random_str(length=16):
    return ''.join(random.choices(string.ascii_letters + string.digits, k=length))


def guess_generator(name, directory, template):
    if os.path.exists(template) and os.path.isdir(template):
        template_dir = os.path.dirname(template)
        return Jinja2Generator(name=name, directory=directory, template=template, template_dir=template_dir)
    else:
        return Jinja2Generator(name=name, directory=directory, template=template)


class AppGenerator:

    ignore_content = """
# Byte-compiled / optimized / DLL files
__pycache__/
*.py[cod]

# C extensions
*.so

# Distribution / packaging
.Python
env/
build/
develop-eggs/
dist/
downloads/
eggs/
lib/
lib64/
parts/
sdist/
var/
*.egg-info/
.installed.cfg
*.egg

# PyInstaller
#  Usually these files are written by a python script from a template
#  before PyInstaller builds the exe, so as to inject date/other infos into it.
*.manifest
*.spec

# Installer logs
pip-log.txt
pip-delete-this-directory.txt

# Unit test / coverage reports
htmlcov/
.tox/
.coverage
.cache
nosetests.xml
coverage.xml

# Translations
*.mo
*.pot

# Django stuff:
*.log

# Sphinx documentation
docs/_build/

# PyBuilder
target/

# IDEs
.idea/

# data
instance/
.DS_store
.env
    """

    def __init__(self, name, directory, template):
        self.name = name
        self.directory = directory or ''
        self.template = template
        self.skip_prompt = False

    @property
    def app_dir(self):
        if self.directory:
            path = os.path.join(self.directory, self.name)
        else:
            path = self.name
        return path

    @property
    def default_additional(self):
        return {}

    def create_app_dir(self, exist_ok=False):
        """
        Create app directory.

        :param bool exist_ok:
        """
        path = self.app_dir
        if not exist_ok and os.path.exists(path):
            raise FileExistsError('App path {} already exists!'.format(path))
        os.makedirs(path, exist_ok=exist_ok)

    def create_ignore(self):
        path = os.path.join(self.app_dir, '.gitignore')
        if not os.path.exists(path):
            with open(path, 'w') as fh:
                fh.write(self.ignore_content)

    def run(self, **kwargs):
        """
        Run generator.
        """
        kwargs = self.pre_process(**kwargs)
        self.generate(**kwargs)
        self.post_process(**kwargs)

    def pre_process(self, **kwargs):
        """
        Pre process before generation.
        """
        if 'skip' in kwargs and kwargs['skip'] is True:
            self.skip_prompt = True
        # create app directory
        self.create_app_dir()
        # generate additional params
        additional = kwargs.get('additional')
        params = self.default_additional
        if additional:
            for a in additional:
                idx = a.find('=')
                if idx > 0:
                    params[a[0:idx]] = a[idx + 1:]
        kwargs['additional_params'] = params
        return kwargs

    def generate(self, **kwargs):
        """
        Generate project from template.
        """
        yield NotImplementedError

    def post_process(self, **kwargs):
        """
        Post process after generation
        """
        self.create_ignore()
        click.echo('Create project {} successfully. Enjoy yourself!'.format(self.app_dir))


class FileGenerator(AppGenerator):

    category = 'file'

    def __init__(self, name, directory, template, template_dir=None):
        super().__init__(name, directory, template)
        self.template_dir = template_dir

    @classmethod
    def get_internal_template_dir(cls):
        return os.path.realpath(os.path.join(os.path.dirname(__file__), 'templates', cls.category))

    @classmethod
    def find_template(cls, template, template_dir=None):
        if not template_dir:
            template_dir = cls.get_internal_template_dir()
        path = os.path.join(template_dir, template)
        if os.path.exists(path):
            return path
        raise FileNotFoundError('Template path {} not found!'.format(path))

    @staticmethod
    def get_rel_path(base_path, path):
        if base_path == path:
            return ''
        elif len(base_path) < len(path):
            return path[len(base_path) + 1:]
        else:
            raise ValueError('Path {} must larger than base path {}!'.format(path, base_path))

    def pre_process(self, **kwargs):
        kwargs = super().pre_process(**kwargs)
        path = self.find_template(template=self.template, template_dir=self.template_dir)
        kwargs['template_path'] = path
        return kwargs

    def generate(self, **kwargs):
        path = kwargs.get('template_path')
        for name, dirs, files in os.walk(path):
            rel_path = self.get_rel_path(path, name)
            # create sub directories
            for d in dirs:
                os.makedirs(os.path.join(self.app_dir, rel_path, d), exist_ok=True)
            # get templates
            if rel_path:
                tmpls = [os.path.join(rel_path, t) for t in files]
            else:
                tmpls = files
            for tmpl in tmpls:
                shutil.copyfile(os.path.join(path, tmpl), os.path.join(self.app_dir, tmpl))


class Jinja2Generator(FileGenerator):

    category = 'jinja2'
    index_list = [
        'https://pypi.org/simple',
        'https://pypi.tuna.tsinghua.edu.cn/simple/',
        'https://mirrors.aliyun.com/pypi/simple',
        'https://pypi.doubanio.com/simple/',
    ]

    @property
    def default_additional(self):
        return {'secret': generate_random_str()}

    @staticmethod
    def get_ignore_filter(ignore_file_pattern, ignore_dir_pattern):
        if not ignore_file_pattern:
            ignore_file_pattern = ['*.pyc', '*.pyo', '*.pyd', '*.egg', '*.log', '*.so', '*.zip', '*.tar', '*.tar.gz']
        if isinstance(ignore_file_pattern, str):
            ignore_file_pattern = ignore_file_pattern.split(',')
        if not ignore_dir_pattern:
            ignore_dir_pattern = ['.git', '__pycache__', '*.egg-info', 'build', 'dist', '.idea']
        if isinstance(ignore_dir_pattern, str):
            ignore_dir_pattern = ignore_dir_pattern.split(',')

        def ignore_file_filter(path):
            for exc in ignore_file_pattern:
                if fnmatch.fnmatch(path, exc):
                    return False
            return True

        def ignore_dir_filter(path):
            paths = path.strip(os.path.sep).split(os.path.sep)
            for exc in ignore_dir_pattern:
                for p in paths:
                    if fnmatch.fnmatch(p, exc):
                        return False
            return True

        return ignore_file_filter, ignore_dir_filter

    def get_context(self, **kwargs):
        context = {
            'name': self.name,
            'directory': self.directory,
            'template': self.template
        }
        context.update(kwargs)
        return context

    def pre_process(self, **kwargs):
        kwargs = super().pre_process(**kwargs)
        # select package index
        if 'pip_index' not in kwargs['additional_params']:
            pip_index = 1
            if not self.skip_prompt:
                pip_index = 0
                msg = ['Select which Python Package Index to use: ', '']
                msg += ['[{}] {}'.format(i + 1, self.index_list[i]) for i in range(len(self.index_list))]
                msg += ['', 'Please input the front number: ']
                while pip_index <= 0 or pip_index > len(self.index_list):
                    pip_index = click.prompt('\n'.join(msg), type=int, default=1)
            kwargs['additional_params']['pip_index'] = self.index_list[pip_index - 1]
        return kwargs

    def generate(self, **kwargs):
        path = kwargs.get('template_path')
        params = kwargs.get('additional_params') or {}
        excludes = kwargs.get('excludes')
        excludes_dir = kwargs.get('excludes_dir')
        excludes_file_filter, excludes_dir_filter = self.get_ignore_filter(excludes, excludes_dir)
        context = self.get_context(template_path=path)
        # load templates
        from jinja2 import Environment, FileSystemLoader
        env = Environment(loader=FileSystemLoader(path))
        for name, dirs, files in os.walk(path):
            rel_path = self.get_rel_path(path, name)
            # filter ignores
            if not excludes_dir_filter(rel_path):
                continue
            files = filter(excludes_file_filter, files)
            # create directory
            os.makedirs(os.path.join(self.app_dir, rel_path), exist_ok=True)
            # get templates
            if rel_path:
                tmpls = [os.path.join(rel_path, t) for t in files]
            else:
                tmpls = files
            for tmpl in tmpls:
                template = env.get_template(tmpl, globals={'context': context})
                target = os.path.join(self.app_dir, tmpl)
                with open(target, 'w') as fh:
                    fh.write(template.render(**params))



from setuptools import setup
from setuptools import find_packages
from flask_board import __version__


with open('README.md') as f:
    LONG_DESCRIPTION = f.read()

setup(
    name='Flask-Board',
    version=__version__,
    description='Build base flask app depends on template.',
    long_description=LONG_DESCRIPTION,
    long_description_content_type='text/markdown',
    url='https://github.com/wwtg99/flask-board',
    author='Wu Wentao',
    author_email='wwtg99@gmail.com',
    license='MIT',
    install_requires=[
        "Werkzeug>=0.15",
        "Jinja2>=2.10.1",
        "flask>=1.0",
        "click>=5.1",
        "python-dotenv>=0.12",
    ],
    include_package_data=True,
    keywords='flask template',
    entry_points={
        'flask.commands': [
            'board=flask_board.commands:cli'
        ],
    },
    classifiers=[
        'Environment :: Console',
        'Intended Audience :: Developers',
        'Intended Audience :: Customer Service',
        'Intended Audience :: System Administrators',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3',
        'Topic :: Software Development',
        'Topic :: Software Development :: Build Tools',
        'Topic :: Software Development :: Debuggers',
        'Topic :: Software Development :: Libraries',
        'Topic :: Software Development :: Libraries :: Python Modules',
        'Topic :: Utilities'
    ],
    packages=find_packages(),
    zip_safe=False,
    platforms='any'
)
